package com.example.nto.controller;

import com.example.nto.entity.Code;
import com.example.nto.entity.CodeBody;
import com.example.nto.entity.Employee;
import com.example.nto.entity.ResponseData;
import com.example.nto.service.EmployeeService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Repository;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.time.LocalDateTime;
import java.util.Optional;

@RestController
public class EmployeeController {
    @Autowired
    private EmployeeService employeeService;

    private ResponseEntity<ResponseData> NotFound() {
        return new ResponseEntity<>(new ResponseData(HttpStatus.UNAUTHORIZED.value(), "Login not found or invalid"), HttpStatus.UNAUTHORIZED);
    }

    private ResponseEntity<ResponseData> BadRequest() {
        return new ResponseEntity<>(new ResponseData(HttpStatus.BAD_REQUEST.value(), "Something went wrong"), HttpStatus.BAD_REQUEST);
    }

    private ResponseEntity<ResponseData> Ok(String message) {
        return new ResponseEntity<>(new ResponseData(HttpStatus.OK.value(), message), HttpStatus.OK);
    }


    @Operation(summary = "Auth employee with login")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200",
                    description = "Auth employee with login success",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = ResponseData.class))}),
            @ApiResponse(responseCode = "401",
                    description = "Login not found or invalid",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = ResponseData.class))}),
            @ApiResponse(responseCode = "400",
                    description = "Something went wrong",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = ResponseData.class))})
    })
    @GetMapping("/api/{login}/auth")
    public ResponseEntity<?> auth(@PathVariable("login") String login) {
        try {
            if (login == null || login.isEmpty() || !employeeService.existsByLogin(login)) {
                return NotFound();
            }

            return Ok("Auth success");
        } catch (Exception e) {
            System.out.println("Exception: " + e.getMessage());
            return BadRequest();
        }
    }


    @Operation(summary = "Get employee info with login")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200",
                    description = "OK",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = Employee.class))}),
            @ApiResponse(responseCode = "401",
                    description = "Login not found or invalid",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = ResponseData.class))}),
            @ApiResponse(responseCode = "400",
                    description = "Something went wrong",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = ResponseData.class))})
    })
    @GetMapping("/api/{login}/info")
    public ResponseEntity<?> info(@PathVariable("login") String login) {
        try {
            if (login == null || login.isEmpty() || login.isBlank() || !employeeService.existsByLogin(login)) {
                return NotFound();
            }

            Optional<Employee> employee = employeeService.findByLogin(login);
            if (employee.isEmpty()) {
                return NotFound();
            }

            return new ResponseEntity<>(employee.get(), HttpStatus.OK);
        } catch (Exception e) {
            System.out.println("Exception: " + e.getMessage());
            return BadRequest();
        }
    }

    @Operation(summary = "Open door with code")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200",
                    description = "Door opened success",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = ResponseData.class))}),
            @ApiResponse(responseCode = "401",
                    description = "Login not found or invalid",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = ResponseData.class))}),
            @ApiResponse(responseCode = "400",
                    description = "Something went wrong",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = ResponseData.class))})
    })
    @PatchMapping("/api/{login}/open")
    public ResponseEntity<?> open(@PathVariable("login") String login, @Valid @RequestBody CodeBody body) {
        try {
            if (login == null || login.isEmpty() || login.isBlank() || !employeeService.existsByLogin(login)) {
                return NotFound();
            }

            Optional<Employee> employee = employeeService.findByLogin(login);
            if (employee.isEmpty()) {
                return NotFound();
            }
            Optional<Code> code = employeeService.findCodeById(employee.get().getId());
            if (code.isEmpty()) {
                return BadRequest();
            }

            long dbValue = 0;
            try {
                dbValue = code.get().getValue();
            } catch (Exception e) {
                employeeService.setCodeEmployee(employee.get().getId(), body.getValue());

                employeeService.setLastVisitEmployee(employee.get().getId(), LocalDateTime.now());
                return Ok("Door opened success");
            }

            if (dbValue != body.getValue()) {
                return BadRequest();
            }

            employeeService.setLastVisitEmployee(employee.get().getId(), LocalDateTime.now());
            return Ok("Door opened success");
        } catch (Exception e) {
            System.out.println("Exception: " + e.getMessage());
            return BadRequest();
        }
    }
}
